@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python -tt
 # -*- coding: utf-8 -*-
 #
 # SoundConverter - GNOME application for converting between audio formats. 
@@ -225,7 +225,8 @@ def vfs_makedirs(path_to_create):
 	for folder in path.split("/"):
 		if not folder:
 			continue
-		uri = uri.append_string(folder)
+		print folder
+		uri = uri.append_string(folder.replace("%2f", "/"))
 		try:
 			gnomevfs.make_directory(uri, 0777)
 		except gnomevfs.FileExistsError:
@@ -246,30 +247,22 @@ def vfs_exists(filename):
 # GStreamer gnomevfssrc helpers
 
 def vfs_encode_filename(filename):
+	filename = filename.replace("%252f", "/")
 	return filename
 
 def file_encode_filename(filename):
 	filename = gnomevfs.get_local_path_from_uri(filename)
 	filename = filename.replace(" ", "\ ");
+	filename = filename.replace("%2f", "/");
 	return filename
 	
 
 def unquote_filename(filename):
 
 	f= urllib.unquote(filename)
-	'''try:
-		# files are normaly in utf-8 ?
-		f = unicode(f, "utf-8")
-	except UnicodeDecodeError:
-		# but sometimes they are badly encoded, this is a failback
-		f = unicode(f, "latin1", "replace")
-	'''
 	return f
 
 
-use_gnomevfs = False
-
-
 def format_tag(tag):
 	if isinstance(tag, list):
 		if len(tag) > 1:
@@ -294,26 +287,36 @@ for element in required_elements:
 		print "required gstreamer element '%s' not found." % element
 		sys.exit(1)
 
-if gst.element_factory_find("gnomevfssrc"):
+use_gnomevfs = False
+
+if gst.element_factory_find("giosrc"):
+	gstreamer_source = "giosrc"
+	gstreamer_sink = "giosink"
+	encode_filename = vfs_encode_filename
+	use_gnomevfs = True
+	print "  using gio"
+elif gst.element_factory_find("gnomevfssrc"):
 	gstreamer_source = "gnomevfssrc"
 	gstreamer_sink = "gnomevfssink"
 	encode_filename = vfs_encode_filename
 	use_gnomevfs = True
+	print "  using deprecated gnomevfssrc"
 else:
 	gstreamer_source = "filesrc"
 	gstreamer_sink = "filesink"
 	encode_filename = file_encode_filename
-	print "  NOT using gnomevfssrc, look for a gnomevfs gstreamer package."
+	print "  not using gnomevfssrc, look for a gnomevfs gstreamer package."
 
 
 encoders = ( 
 	("flacenc",		"FLAC"), 
 	("wavenc",		"WAV"),
-	("vorbisenc", "Ogg Vorbis"),
+	("vorbisenc",   "Ogg Vorbis"),
 	("oggmux",		"Ogg Vorbis"),
 	("id3v2mux",	"MP3 Tags"),
 	("xingmux",		""),
-	("lame",			"MP3"))
+	("lame",		"MP3"),
+	("faac",        "AAC"))
 
 for encoder, name in encoders:
 	have_it = True
@@ -509,6 +512,8 @@ class TargetNameGenerator:
 		}
 		for key in sound_file.keys():
 			dict[key] = sound_file[key]
+			if isinstance(dict[key], basestring):
+				dict[key] = dict[key].replace("/", "-")
 		
 		pattern = os.path.join(self.subfolders, self.basename + self.suffix)
 		result = pattern % dict
@@ -1123,6 +1128,7 @@ class Converter(Decoder):
 			"audio/x-flac": self.add_flac_encoder,
 			"audio/x-wav": self.add_wav_encoder,
 			"audio/mpeg": self.add_mp3_encoder,
+			"audio/x-m4a": self.add_aac_encoder,
 		}
 
 		self.add_command("audioconvert")
@@ -1153,7 +1159,8 @@ class Converter(Decoder):
 				dialog.hide()
 				return
 	
-		self.add_command('gnomevfssink location=%s' % uri)
+		self.add_command('%s location=%s' % (
+			gstreamer_sink, encode_filename(self.output_filename)))
 		if self.overwrite and vfs_exists(self.output_filename):
 			log("overwriting '%s'" % self.output_filename)
 			vfs_unlink(self.output_filename)
@@ -1229,6 +1236,9 @@ class Converter(Decoder):
 		
 		return cmd
 
+	def add_aac_encoder(self):
+		return "faac profile=2 ! ffmux_mp4"
+
 class FileList:
 	"""List of files added by the user."""
 
@@ -1547,6 +1557,7 @@ class PreferencesDialog:
 						"audio/x-flac"	: (have_flacenc, "output_mime_type_flac"),
 						"audio/x-wav"		: (have_wavenc, "output_mime_type_wav"),
 						"audio/mpeg"		: (have_lame, "output_mime_type_mp3"),
+						"audio/x-m4a"       : (have_faac, "output_mime_type_aac"),
 					}
 
 		# desactivate output if encoder plugin is not present
@@ -1689,6 +1700,7 @@ class PreferencesDialog:
 						"audio/x-flac": ".flac",
 						"audio/x-wav": ".wav",
 						"audio/mpeg": ".mp3",
+                        "audio/x-m4a": ".m4a",
 					}.get(output_type, None)
 
 		generator = TargetNameGenerator()
@@ -1696,9 +1708,9 @@ class PreferencesDialog:
 		generator.set_target_suffix(output_suffix)
 		if not self.get_int("same-folder-as-input"):
 			generator.set_folder(self.get_string("selected-folder"))
-			if self.get_int("create-subfolders"):
-				generator.set_subfolder_pattern(
-					self.get_subfolder_pattern())
+		if self.get_int("create-subfolders"):
+			generator.set_subfolder_pattern(
+				self.get_subfolder_pattern())
 		generator.set_basename_pattern(self.get_basename_pattern())
 		if for_display:
 			generator.set_replace_messy_chars(False)
@@ -1844,6 +1856,7 @@ class PreferencesDialog:
 						"audio/mpeg": 1,
 						"audio/x-flac": 2,
 						"audio/x-wav": 3,
+						"audio/x-m4a": 3,
 		}
 		self.quality_tabs.set_current_page(tabs[mime_type])
 
@@ -1863,6 +1876,10 @@ class PreferencesDialog:
 		if button.get_active():
 			self.change_mime_type("audio/mpeg")
 
+	def on_output_mime_type_aac_toggled(self, button):
+		if button.get_active():
+			self.change_mime_type("audio/x-m4a")
+
 	def on_vorbis_quality_changed(self, combobox):
 		quality = (0,0.2,0.4,0.6,0.8)
 		self.set_float("vorbis-quality", quality[combobox.get_active()])
@@ -2536,7 +2553,7 @@ def cli_convert_main(input_files):
 	queue.run()
 	while queue.is_running():
 		t = queue.get_current_task()
-		if not get_option("quiet"):
+		if t and not get_option("quiet"):
 			if previous_filename != t.sound_file.get_filename_for_display():
 				if previous_filename:
 					print _("%s: OK") % previous_filename
